@@ -49,10 +49,10 @@ inline bool keeping_cutbuffer(void)
 void cut_line(void)
 {
     if (openfile->current != openfile->filebot)
-	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current, 0,
+	extract_buffer(&cutbuffer, &cutbottom, openfile->current, 0,
 		openfile->current->next, 0);
     else
-	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current, 0,
+	extract_buffer(&cutbuffer, &cutbottom, openfile->current, 0,
 		openfile->current, strlen(openfile->current->data));
     openfile->placewewant = 0;
 }
@@ -68,7 +68,7 @@ void cut_marked(bool *right_side_up)
     mark_order((const filestruct **)&top, &top_x,
 		(const filestruct **)&bot, &bot_x, right_side_up);
 
-    move_to_filestruct(&cutbuffer, &cutbottom, top, top_x, bot, bot_x);
+    extract_buffer(&cutbuffer, &cutbottom, top, top_x, bot, bot_x);
     openfile->placewewant = xplustabs();
 }
 
@@ -86,14 +86,14 @@ void cut_to_eol(void)
 	/* If we're not at the end of the line, move all the text from
 	 * the current position up to it, not counting the newline at
 	 * the end, into the cutbuffer. */
-	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current,
+	extract_buffer(&cutbuffer, &cutbottom, openfile->current,
 		openfile->current_x, openfile->current, data_len);
     else if (openfile->current != openfile->filebot) {
 	/* If we're at the end of the line, and it isn't the last line
 	 * of the file, move all the text from the current position up
 	 * to the beginning of the next line, i.e. the newline at the
 	 * end, into the cutbuffer. */
-	move_to_filestruct(&cutbuffer, &cutbottom, openfile->current,
+	extract_buffer(&cutbuffer, &cutbottom, openfile->current,
 		openfile->current_x, openfile->current->next, 0);
 	openfile->placewewant = xplustabs();
     }
@@ -103,7 +103,7 @@ void cut_to_eol(void)
  * file into the cutbuffer. */
 void cut_to_eof(void)
 {
-    move_to_filestruct(&cutbuffer, &cutbottom,
+    extract_buffer(&cutbuffer, &cutbottom,
 		openfile->current, openfile->current_x,
 		openfile->filebot, strlen(openfile->filebot->data));
 }
@@ -172,10 +172,10 @@ void do_cut_text(bool copy_text, bool cut_till_eof)
 	if (cutbuffer != NULL) {
 	    if (cb_save != NULL) {
 		cb_save->data += cb_save_len;
-		copy_from_filestruct(cb_save);
+		copy_from_buffer(cb_save);
 		cb_save->data -= cb_save_len;
 	    } else
-		copy_from_filestruct(cutbuffer);
+		copy_from_buffer(cutbuffer);
 
 	    /* If the copied region was marked forward, put the new desired
 	     * x position at its end; otherwise, leave it at its beginning. */
@@ -194,7 +194,7 @@ void do_cut_text(bool copy_text, bool cut_till_eof)
     refresh_needed = TRUE;
 
 #ifndef DISABLE_COLOR
-    reset_multis(openfile->current, FALSE);
+    check_the_multis(openfile->current);
 #endif
 
 #ifdef DEBUG
@@ -223,7 +223,7 @@ void do_copy_text(void)
     static struct filestruct *next_contiguous_line = NULL;
     bool mark_set = openfile->mark_set;
 
-    /* Remember the current view port and cursor position. */
+    /* Remember the current viewport and cursor position. */
     ssize_t is_edittop_lineno = openfile->edittop->lineno;
     ssize_t is_current_lineno = openfile->current->lineno;
     size_t is_current_x = openfile->current_x;
@@ -237,7 +237,7 @@ void do_copy_text(void)
     next_contiguous_line = (mark_set ? NULL : openfile->current);
 
     if (mark_set) {
-	/* Restore the view port and cursor position. */
+	/* Restore the viewport and cursor position. */
 	openfile->edittop = fsfromline(is_edittop_lineno);
 	openfile->current = fsfromline(is_current_lineno);
 	openfile->current_x = is_current_x;
@@ -268,7 +268,7 @@ void do_uncut_text(void)
 
     /* Add a copy of the text in the cutbuffer to the current filestruct
      * at the current cursor position. */
-    copy_from_filestruct(cutbuffer);
+    copy_from_buffer(cutbuffer);
 
 #ifndef NANO_TINY
     update_undo(PASTE);
@@ -291,7 +291,7 @@ void do_uncut_text(void)
     refresh_needed = TRUE;
 
 #ifndef DISABLE_COLOR
-    reset_multis(openfile->current, FALSE);
+    check_the_multis(openfile->current);
 #endif
 
 #ifdef DEBUG
