@@ -29,8 +29,8 @@
 
 /* Global variables. */
 #ifndef NANO_TINY
-volatile sig_atomic_t sigwinch_counter = 0;
-	/* Is incremented by the handler whenever a SIGWINCH occurs. */
+volatile sig_atomic_t the_window_resized = FALSE;
+	/* Set to TRUE by the handler whenever a SIGWINCH occurs. */
 #endif
 
 #ifdef __linux__
@@ -45,20 +45,20 @@ bool shift_held;
 bool focusing = TRUE;
 	/* Whether an update of the edit window should center the cursor. */
 
+bool as_an_at = TRUE;
+	/* Whether a 0x0A byte should be shown as a ^@ instead of a ^J. */
+
 int margin = 0;
 	/* The amount of space reserved at the left for line numbers. */
 int editwincols = -1;
 	/* The number of usable columns in the edit window: COLS - margin. */
-#ifdef ENABLE_LINENUMBERS
-int last_drawn_line = 0;
-        /* The line number of the last drawn line. */
-int last_line_y;
-        /* The y coordinate of the last drawn line. */
-#endif
 
 message_type lastmessage = HUSH;
 	/* Messages of type HUSH should not overwrite type MILD nor ALERT. */
 
+filestruct *pletion_line = NULL;
+	/* The line where the last completion was found, if any. */
+
 int controlleft, controlright, controlup, controldown;
 #ifndef NANO_TINY
 int shiftcontrolleft, shiftcontrolright, shiftcontrolup, shiftcontroldown;
@@ -94,8 +94,8 @@ WINDOW *bottomwin;
 	 * messages, the statusbar prompt, and a list of shortcuts. */
 int editwinrows = 0;
 	/* How many rows does the edit window take up? */
-int maxrows = 0;
-	/* How many usable lines there are (due to soft wrapping). */
+int maxlines = 0;
+	/* How many file lines can be shown (due to soft wrapping). */
 
 filestruct *cutbuffer = NULL;
 	/* The buffer where we store cut text. */
@@ -388,18 +388,17 @@ const sc *first_sc_for(int menu, void (*func)(void))
     return NULL;
 }
 
-/* Return the given menu's first shortcut sequence, or the default value
- * (2nd arg).  Assumes currmenu for the menu to check. */
-int sc_seq_or(void (*func)(void), int defaultval)
+/* Return the first keycode that is bound to the given function in the
+ * current menu, if any; otherwise, return the given default value. */
+int the_code_for(void (*func)(void), int defaultval)
 {
     const sc *s = first_sc_for(currmenu, func);
 
-    if (s) {
-	meta_key = s->meta;
-	return s->keycode;
-    }
-    /* else */
-    return defaultval;
+    if (s == NULL)
+	return defaultval;
+
+    meta_key = s->meta;
+    return s->keycode;
 }
 
 /* Return a pointer to the function that is bound to the given key. */
@@ -531,9 +530,6 @@ void shortcut_init(void)
 	N_("Copy the current line and store it in the cutbuffer");
     const char *nano_indent_msg = N_("Indent the current line");
     const char *nano_unindent_msg = N_("Unindent the current line");
-#ifdef ENABLE_COMMENT
-    const char *nano_comment_msg = N_("Comment/uncomment the current line or marked lines");
-#endif
     const char *nano_undo_msg = N_("Undo the last operation");
     const char *nano_redo_msg = N_("Redo the last undone operation");
 #endif
@@ -591,6 +587,13 @@ void shortcut_init(void)
 	N_("Refresh (redraw) the current screen");
     const char *nano_suspend_msg =
 	N_("Suspend the editor (if suspension is enabled)");
+#ifdef ENABLE_WORDCOMPLETION
+    const char *nano_completion_msg = N_("Try and complete the current word");
+#endif
+#ifdef ENABLE_COMMENT
+    const char *nano_comment_msg =
+	N_("Comment/uncomment the current line or marked lines");
+#endif
 #ifndef NANO_TINY
     const char *nano_savefile_msg = N_("Save file without prompting");
     const char *nano_findprev_msg = N_("Search next occurrence backward");
@@ -925,6 +928,10 @@ void shortcut_init(void)
     add_to_funcs(do_suspend_void, MMAIN,
 	N_("Suspend"), IFSCHELP(nano_suspend_msg), BLANKAFTER, VIEW);
 
+#ifdef ENABLE_WORDCOMPLETION
+    add_to_funcs(complete_a_word, MMAIN,
+	N_("Complete"), IFSCHELP(nano_completion_msg), TOGETHER, NOVIEW);
+#endif
 #ifdef ENABLE_COMMENT
     add_to_funcs(do_comment, MMAIN,
 	N_("Comment Lines"), IFSCHELP(nano_comment_msg), BLANKAFTER, NOVIEW);
@@ -1096,6 +1103,9 @@ void shortcut_init(void)
     add_to_sclist(MMAIN, "M-U", 0, do_undo, 0);
     add_to_sclist(MMAIN, "M-E", 0, do_redo, 0);
 #endif
+#ifdef ENABLE_WORDCOMPLETION
+    add_to_sclist(MMAIN, "^]", 0, complete_a_word, 0);
+#endif
 #ifdef ENABLE_COMMENT
     add_to_sclist(MMAIN, "M-3", 0, do_comment, 0);
 #endif
@@ -1443,6 +1453,10 @@ sc *strtosc(const char *input)
     else if (!strcasecmp(input, "comment"))
 	s->scfunc = do_comment;
 #endif
+#ifdef ENABLE_WORDCOMPLETION
+    else if (!strcasecmp(input, "complete"))
+	s->scfunc = complete_a_word;
+#endif
 #ifndef NANO_TINY
     else if (!strcasecmp(input, "indent"))
 	s->scfunc = do_indent_void;
