@@ -33,6 +33,10 @@
 
 #ifndef DISABLE_NANORC
 
+#ifndef RCFILE_NAME
+#define RCFILE_NAME ".nanorc"
+#endif
+
 static const rcoption rcopts[] = {
     {"boldtext", BOLD_TEXT},
 #ifdef ENABLE_LINENUMBERS
@@ -526,35 +530,40 @@ void parse_binding(char *ptr, bool dobind)
     free(keycopy);
 }
 
+/* Verify that the given file is not a folder nor a device. */
+bool is_good_file(char *file)
+{
+    struct stat rcinfo;
+
+    /* If the thing exists, it may not be a directory nor a device. */
+    if (stat(file, &rcinfo) != -1 && (S_ISDIR(rcinfo.st_mode) ||
+		S_ISCHR(rcinfo.st_mode) || S_ISBLK(rcinfo.st_mode))) {
+	rcfile_error(S_ISDIR(rcinfo.st_mode) ? _("\"%s\" is a directory") :
+					_("\"%s\" is a device file"), file);
+	return FALSE;
+    } else
+	return TRUE;
+}
+
 #ifndef DISABLE_COLOR
 /* Read and parse one included syntax file. */
 static void parse_one_include(char *file)
 {
-    struct stat rcinfo;
     FILE *rcstream;
 
-    /* Can't get the specified file's full path because it may screw up
-     * our cwd depending on the parent directories' permissions (see
-     * Savannah bug #25297). */
-
     /* Don't open directories, character files, or block files. */
-    if (stat(file, &rcinfo) != -1) {
-	if (S_ISDIR(rcinfo.st_mode) || S_ISCHR(rcinfo.st_mode) ||
-		S_ISBLK(rcinfo.st_mode)) {
-	    rcfile_error(S_ISDIR(rcinfo.st_mode) ?
-		_("\"%s\" is a directory") :
-		_("\"%s\" is a device file"), file);
-	}
-    }
+    if (!is_good_file(file))
+	return;
 
-    /* Open the new syntax file. */
-    if ((rcstream = fopen(file, "rb")) == NULL) {
-	rcfile_error(_("Error reading %s: %s"), file,
-		strerror(errno));
+    /* Open the included syntax file. */
+    rcstream = fopen(file, "rb");
+
+    if (rcstream == NULL) {
+	rcfile_error(_("Error reading %s: %s"), file, strerror(errno));
 	return;
     }
 
-    /* Use the name and line number position of the new syntax file
+    /* Use the name and line number position of the included syntax file
      * while parsing it, so we can know where any errors in it are. */
     nanorc = file;
     lineno = 0;
@@ -585,8 +594,7 @@ void parse_includes(char *ptr)
 	for (i = 0; i < files.gl_pathc; ++i)
 	    parse_one_include(files.gl_pathv[i]);
     } else
-	rcfile_error(_("Error expanding %s: %s"), option,
-		strerror(errno));
+	rcfile_error(_("Error expanding %s: %s"), option, strerror(errno));
 
     globfree(&files);
     free(expanded);
@@ -640,7 +648,7 @@ void parse_colors(char *ptr, int rex_flags)
 {
     short fg, bg;
     bool bright = FALSE;
-    char *fgstr;
+    char *item;
 
     assert(ptr != NULL);
 
@@ -656,9 +664,9 @@ void parse_colors(char *ptr, int rex_flags)
 	return;
     }
 
-    fgstr = ptr;
+    item = ptr;
     ptr = parse_next_word(ptr);
-    if (!parse_color_names(fgstr, &fg, &bg, &bright))
+    if (!parse_color_names(item, &fg, &bg, &bright))
 	return;
 
     if (*ptr == '\0') {
@@ -688,12 +696,16 @@ void parse_colors(char *ptr, int rex_flags)
 	    continue;
 	}
 
-	fgstr = ++ptr;
+	item = ++ptr;
 	ptr = parse_next_regex(ptr);
 	if (ptr == NULL)
 	    break;
 
-	goodstart = nregcomp(fgstr, rex_flags);
+	if (*item == '\0') {
+	    rcfile_error(N_("Empty regex string"));
+	    goodstart = FALSE;
+	} else
+	    goodstart = nregcomp(item, rex_flags);
 
 	/* If the starting regex is valid, initialize a new color struct,
 	 * and hook it in at the tail of the linked list. */
@@ -705,7 +717,7 @@ void parse_colors(char *ptr, int rex_flags)
 	    newcolor->bright = bright;
 	    newcolor->rex_flags = rex_flags;
 
-	    newcolor->start_regex = mallocstrcpy(NULL, fgstr);
+	    newcolor->start_regex = mallocstrcpy(NULL, item);
 	    newcolor->start = NULL;
 
 	    newcolor->end_regex = NULL;
@@ -738,19 +750,24 @@ void parse_colors(char *ptr, int rex_flags)
 	    continue;
 	}
 
-	fgstr = ++ptr;
+	item = ++ptr;
 	ptr = parse_next_regex(ptr);
 	if (ptr == NULL)
 	    break;
 
+	if (*item == '\0') {
+	    rcfile_error(N_("Empty regex string"));
+	    continue;
+	}
+
 	/* If the start regex was invalid, skip past the end regex
 	 * to stay in sync. */
 	if (!goodstart)
 	    continue;
 
 	/* If it's valid, save the ending regex string. */
-	if (nregcomp(fgstr, rex_flags))
-	    newcolor->end_regex = mallocstrcpy(NULL, fgstr);
+	if (nregcomp(item, rex_flags))
+	    newcolor->end_regex = mallocstrcpy(NULL, item);
 
 	/* Lame way to skip another static counter. */
 	newcolor->id = live_syntax->nmultis;
@@ -904,8 +921,8 @@ void pick_up_name(const char *kind, char *ptr, char **storage)
 }
 #endif /* !DISABLE_COLOR */
 
-/* Check whether the user has unmapped every shortcut for a
- * sequence we consider 'vital', like the exit function. */
+/* Verify that the user has not unmapped every shortcut for a
+ * function that we consider 'vital' (such as "Exit"). */
 static void check_vitals_mapped(void)
 {
     subnfunc *f;
@@ -932,13 +949,9 @@ static void check_vitals_mapped(void)
 }
 
 /* Parse the rcfile, once it has been opened successfully at rcstream,
- * and close it afterwards.  If syntax_only is TRUE, only allow the file
- * to contain color syntax commands. */
-void parse_rcfile(FILE *rcstream
-#ifndef DISABLE_COLOR
-	, bool syntax_only
-#endif
-	)
+ * and close it afterwards.  If syntax_only is TRUE, allow the file to
+ * to contain only color syntax commands. */
+void parse_rcfile(FILE *rcstream, bool syntax_only)
 {
     char *buf = NULL;
     ssize_t len;
@@ -1115,7 +1128,7 @@ void parse_rcfile(FILE *rcstream
 #endif
 	/* Make sure the option argument is a valid multibyte string. */
 	if (!is_valid_mbstring(option)) {
-	    rcfile_error(N_("Option is not a valid multibyte string"));
+	    rcfile_error(N_("Argument is not a valid multibyte string"));
 	    continue;
 	}
 
@@ -1144,7 +1157,8 @@ void parse_rcfile(FILE *rcstream
 				option);
 		wrap_at = -CHARS_FROM_EOL;
 	    } else
-		free(option);
+		UNSET(NO_WRAP);
+	    free(option);
 	} else
 #endif
 #ifndef NANO_TINY
@@ -1205,8 +1219,8 @@ void parse_rcfile(FILE *rcstream
 		rcfile_error(N_("Requested tab size \"%s\" is invalid"),
 				option);
 		tabsize = -1;
-	    } else
-		free(option);
+	    }
+	    free(option);
 	} else
 	    assert(FALSE);
     }
@@ -1223,45 +1237,43 @@ void parse_rcfile(FILE *rcstream
     fclose(rcstream);
     lineno = 0;
 
-    check_vitals_mapped();
     return;
 }
 
-/* The main rcfile function.  It tries to open the system-wide rcfile,
- * followed by the current user's rcfile. */
-void do_rcfile(void)
+/* Read and interpret one of the two nanorc files. */
+void parse_one_nanorc(void)
 {
-    struct stat rcinfo;
     FILE *rcstream;
 
-    nanorc = mallocstrcpy(nanorc, SYSCONFDIR "/nanorc");
-
-    /* Don't open directories, character files, or block files. */
-    if (stat(nanorc, &rcinfo) != -1) {
-	if (S_ISDIR(rcinfo.st_mode) || S_ISCHR(rcinfo.st_mode) ||
-		S_ISBLK(rcinfo.st_mode))
-	    rcfile_error(S_ISDIR(rcinfo.st_mode) ?
-		_("\"%s\" is a directory") :
-		_("\"%s\" is a device file"), nanorc);
-    }
+    /* Don't try to open directories nor devices. */
+    if (!is_good_file(nanorc))
+	return;
 
 #ifdef DEBUG
-    fprintf(stderr, "Parsing file \"%s\"\n", nanorc);
+    fprintf(stderr, "Going to parse file \"%s\"\n", nanorc);
 #endif
 
-    /* Try to open the system-wide nanorc. */
     rcstream = fopen(nanorc, "rb");
-    if (rcstream != NULL)
-	parse_rcfile(rcstream
-#ifndef DISABLE_COLOR
-		, FALSE
-#endif
-		);
 
+    /* If opening the file succeeded, parse it.  Otherwise, only
+     * complain if the file actually exists. */
+    if (rcstream != NULL)
+	parse_rcfile(rcstream, FALSE);
+    else if (errno != ENOENT)
+	rcfile_error(N_("Error reading %s: %s"), nanorc, strerror(errno));
+}
+
+/* First read the system-wide rcfile, then the user's rcfile. */
+void do_rcfiles(void)
+{
+    nanorc = mallocstrcpy(nanorc, SYSCONFDIR "/nanorc");
+
+    /* Process the system-wide nanorc. */
+    parse_one_nanorc();
+
+    /* When configured with --disable-wrapping-as-root, turn wrapping off
+     * for root, so that only root's .nanorc or --fill can turn it on. */
 #ifdef DISABLE_ROOTWRAPPING
-    /* We've already read SYSCONFDIR/nanorc, if it's there.  If we're
-     * root, and --disable-wrapping-as-root is used, turn wrapping off
-     * now. */
     if (geteuid() == NANO_ROOT_UID)
 	SET(NO_WRAP);
 #endif
@@ -1271,38 +1283,16 @@ void do_rcfile(void)
     if (homedir == NULL)
 	rcfile_error(N_("I can't find my home directory!  Wah!"));
     else {
-#ifndef RCFILE_NAME
-#define RCFILE_NAME ".nanorc"
-#endif
 	nanorc = charealloc(nanorc, strlen(homedir) + strlen(RCFILE_NAME) + 2);
 	sprintf(nanorc, "%s/%s", homedir, RCFILE_NAME);
 
-	/* Don't open directories, character files, or block files. */
-	if (stat(nanorc, &rcinfo) != -1) {
-	    if (S_ISDIR(rcinfo.st_mode) || S_ISCHR(rcinfo.st_mode) ||
-			S_ISBLK(rcinfo.st_mode))
-		rcfile_error(S_ISDIR(rcinfo.st_mode) ?
-			_("\"%s\" is a directory") :
-			_("\"%s\" is a device file"), nanorc);
-	}
-
-	/* Try to open the current user's nanorc. */
-	rcstream = fopen(nanorc, "rb");
-	if (rcstream == NULL) {
-	    /* Don't complain about the file's not existing. */
-	    if (errno != ENOENT)
-		rcfile_error(N_("Error reading %s: %s"), nanorc,
-				strerror(errno));
-	} else
-	    parse_rcfile(rcstream
-#ifndef DISABLE_COLOR
-		, FALSE
-#endif
-		);
+	/* Process the current user's nanorc. */
+	parse_one_nanorc();
     }
 
+    check_vitals_mapped();
+
     free(nanorc);
-    nanorc = NULL;
 
     if (errors && !ISSET(QUIET)) {
 	errors = FALSE;
